package fr.itsonus.bousoleplussbackend.security.services;

import fr.itsonus.bousoleplussbackend.domain.auth.model.RefreshToken;
import fr.itsonus.bousoleplussbackend.domain.auth.spi.RefreshTokenCacheRepository;
import fr.itsonus.bousoleplussbackend.domain.auth.spi.UserCacheRepository;
import fr.itsonus.bousoleplussbackend.exception.TokenRefreshException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.Instant;
import java.util.Optional;
import java.util.UUID;

@Service
public class RefreshTokenService {
    @Value("${security.jwt.refresh-expiration}")
    private Long refreshTokenDurationMs;

    private final RefreshTokenCacheRepository refreshTokenCacheRepository;

    private final UserCacheRepository userCacheRepository;

    public RefreshTokenService(final RefreshTokenCacheRepository refreshTokenCacheRepository,
                               final UserCacheRepository userCacheRepository) {
        this.refreshTokenCacheRepository = refreshTokenCacheRepository;
        this.userCacheRepository = userCacheRepository;
    }

    public Optional<RefreshToken> findByToken(String token) {
        return refreshTokenCacheRepository.findByToken(token);
    }

    public RefreshToken createOrUpdateRefreshToken(Long userId) {
        var user = userCacheRepository.findById(userId).orElseThrow();
        var maybeRefreshToken = refreshTokenCacheRepository.findByUser(user);
        RefreshToken refreshToken = maybeRefreshToken.orElseGet(() -> RefreshToken.builder().user(user).build())
                .toBuilder()
                .expiryDate(Instant.now().plusMillis(refreshTokenDurationMs))
                .token(UUID.randomUUID().toString())
                .build();

        return refreshTokenCacheRepository.save(refreshToken);
    }

    public RefreshToken verifyExpiration(RefreshToken token) {
        if (token.getExpiryDate().compareTo(Instant.now()) < 0) {
            refreshTokenCacheRepository.delete(token);
            throw new TokenRefreshException(token.getToken(),
                    "Refresh token was expired. Please make a new signin request");
        }
        return token;
    }

    @Transactional
    public void deleteByUserId(Long userId) {
        refreshTokenCacheRepository.deleteByUserId(userId);
    }
}
